import os
import json
from numbers import Number
from fgmk import print_error

try:
  basestring
except NameError:
  basestring = (str, bytes)

class BaseFormat:
    def __init__(self, filename=""):
        self.jsonTree = {}
        self.filename = filename

    def new(self):
        self.jsonTree = {}

    def save(self, filename=""):
        if(filename != ""):
            self.filename = filename

        writesafe(self.jsonTree, self.filename)

    def exportJS(self, charsn):
        writefile.writesafe(self.jsonTree, f, self.jsvarname )
        #TODO set the correct varname here

    def load(self, filename=""):
        if(filename != ""):
            self.filename = filename

        f = open(self.filename, "r")
        self.jsonTree = json.load(f)
        f.close()

def ordered(obj):
    """
    Recursive order a unordered dict. Use for comparing dicts.
    """
    if isinstance(obj, dict):
        return sorted((k, ordered(v)) for k, v in obj.items())
    else:
        return obj

def isJsonEqual(jsonA,jsonB):
    """
    Check if two json tree (dicts) are equal.

    Args:
        jsonA dict: a dict or jsontree.
        jsonB dict: another dict or jsontree.

    Returns:
        bool. True if equal, False if not.
    """
    return ordered(jsonA) == ordered(jsonB)


def writesafe(data, fname, varname=None):
    """
    Write a dict to a file as Json safely.

    Write the content of a dict to a file with name fname~ and if it's content
    is equal to data, renames fname~ to just fname. This function can optionaly
    generate a JS object if varname is not None.

    Args:
        data dict: a dict you wish to write to a file.
        fname str: the file name.
    """
    tempfile = fname+'~'
    try:
        f = open(tempfile, 'w+')
        try:
            if(varname==None):
                fwriteKeyVals(data, f)
            else:
                f.write("var " + self.levelName + "= {\n")
                fwriteKeyValsJS(data,f)
                f.write("};")
        finally:
            f.flush()
            os.fsync(f.fileno())
            f.close()
            if(varname==None):
                try:
                    f = open(tempfile, 'r')
                    tempjsontree = json.load(f)
                    if isJsonEqual(data,tempjsontree):
                        f.close()
                        os.replace(tempfile,fname)
                    else:
                        print_error.printe("written file not equal to data")
                except:
                    print_error.printe("error when checking file")
            else:
                os.replace(tempfile,fname)
    except:
        print_error.printe('error when opening file')



def fwriteKeyVals(data, f, indent=0):
    """
    Recursively write a JSON from dict data to an opened file f.
    """

    if isinstance(data, list):
        try:
            gotdata = isinstance(data[0], list)
        except IndexError:
            gotdata = False

        if gotdata:

            f.write("\n" + "    " * indent + "[")
            for i in range(len(data)):
                if (i == 0):
                    f.write("[")
                else:
                    f.write("    " * indent + " [")
                for j in range(len(data[0])):
                    if isinstance(data[i][j], Number):
                        f.write("%3d" % data[i][j])
                    elif isinstance(data[i][j], bool):
                        f.write("%3d" % data[i][j])
                    elif isinstance(data[i][j], basestring):
                        dataListLf = data[i][j].split("\n")
                        dataToWrite = dataListLf[0]
                        for line in dataListLf[1:]:
                            dataToWrite += '\\n' + line

                        f.write("\"" + dataToWrite + "\"")
                    else:
                        f.write(dataToWrite)
                    f.write(",") if j != len(
                        data[0]) - 1 else (f.write("],") if i != len(data) - 1 else f.write("]"))
                f.write("\n") if i != len(data) - 1 else f.write("]")
        else:
            try:
                gotdata = data[0]
            except IndexError:
                gotdata = 'False'

            if gotdata is not 'False':
                f.write(" [")
                for i in range(len(data)):
                    if isinstance(data[i], Number):
                        f.write("%3d" % data[i])
                    elif isinstance(data[i], bool):
                        f.write("%3d" % data[i][j])
                    elif isinstance(data[i], basestring):
                        dataListLf = data[i].split("\n")
                        dataToWrite = dataListLf[0]
                        for line in dataListLf[1:]:
                            dataToWrite += '\\n' + line

                        f.write("\"" + dataToWrite + "\"")
                    else:
                        f.write(dataToWrite)
                    f.write(",") if i != len(data) - 1 else f.write("]")
            else:
                f.write(" []")

    elif isinstance(data, dict):
        f.write("\n" + "    " * indent + "{")
        for k, v in data.items():
            f.write("\n" + "    " * indent + "\"" + k + "\"" + ": ")
            fwriteKeyVals(v, f, indent + 1)
            if(list(data)[-1] != k):
                f.write(",")
        f.write("\n" + "    " * indent + "}")
    elif isinstance(data, bool):
        f.write("%3d" % data)
    elif isinstance(data, basestring):
        f.write("\"" + data + "\"")
    else:
        f.write( str(data) )


def fwriteKeyValsJS(data, f, indent=0):
    """
    Recursively write a JS object from dict data to an opened file f.
    """

    if isinstance(data, list):
        try:
            gotdata = isinstance(data[0], list)
        except IndexError:
            gotdata = False

        if gotdata:

            f.write("\n" + "    " * indent + "[")
            for i in range(len(data)):
                if (i == 0):
                    f.write("[")
                else:
                    f.write("    " * indent + " [")
                for j in range(len(data[0])):
                    if isinstance(data[i][j], Number):
                        f.write("%3d" % data[i][j])
                    else:
                        dataListLf = data[i][j].split("\n")
                        dataToWrite = dataListLf[0]
                        for line in dataListLf[1:]:
                            dataToWrite += '\\n' + line

                        f.write("\"" + dataToWrite + "\"")

                    f.write(",") if j != len(
                        data[0]) - 1 else (f.write("],") if i != len(data) - 1 else f.write("]"))
                f.write("\n") if i != len(data) - 1 else f.write("]")
        else:
            try:
                gotdata = data[0]
            except IndexError:
                gotdata = 'False'

            if gotdata is not 'False':
                f.write(" [")
                for i in range(len(data)):
                    if isinstance(data[i], Number):
                        f.write("%3d" % data[i])
                    else:
                        dataListLf = data[i].split("\n")
                        dataToWrite = dataListLf[0]
                        for line in dataListLf[1:]:
                            dataToWrite += '\\n' + line

                        f.write("\"" + dataToWrite + "\"")
                    f.write(",") if i != len(data) - 1 else f.write("]")
            else:
                f.write(" [\"\"]")

    elif isinstance(data, dict):
        if(indent):
            f.write("\n" + "    " * indent + "{")

        for k, v in data.items():
            f.write("\n" + "    " * indent + "" + k + "" + ": ")
            fwriteKeyValsJS(v, f, indent + 1)
            if(list(data)[-1] != k):
                f.write(",")

        if(indent):
            f.write("\n" + "    " * indent + "}")

    else:
        f.write("\"" + data + "\"")
